@@ -1,28 +1,3 @@
-/*
-  Racket
-  Copyright (c) 2004-2018 PLT Design Inc.
-  Copyright (c) 1995-2001 Matthew Flatt
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301 USA.
-
-  libscheme
-  Copyright (c) 1994 Brent Benson
-  All rights reserved.
-*/
-
 #include "schpriv.h"
 #include "schminc.h"
 #include "schmach.h"
@@ -181,6 +156,7 @@ Scheme_Env *scheme_basic_env()
 
   scheme_init_finalization();
 
+  scheme_init_hash_tree();
   scheme_init_portable_case();
   scheme_init_compenv();
   scheme_init_letrec_check();
@@ -303,9 +279,7 @@ static void init_startup_env(void)
   MZTIMEIT(eval, scheme_init_eval(env));
   MZTIMEIT(struct, scheme_init_struct(env));
   MZTIMEIT(error, scheme_init_error(env));
-#ifndef NO_SCHEME_EXNS
   MZTIMEIT(exn, scheme_init_exn(env));
-#endif
   MZTIMEIT(process, scheme_init_thread(env));
   scheme_init_port_wait();
   scheme_init_inspector();
@@ -347,9 +321,10 @@ static void init_startup_env(void)
   
 #if USE_COMPILED_STARTUP
   if (builtin_ref_counter != EXPECTED_PRIM_COUNT) {
-    printf("Primitive count %d doesn't match expected count %d\n"
-	   "Turn off USE_COMPILED_STARTUP in src/schminc.h\n",
-	   builtin_ref_counter, EXPECTED_PRIM_COUNT);
+    fprintf(stderr,
+            "Primitive count %d doesn't match expected count %d\n"
+            "Update the count in src/schminc.h and bump the version in src/schvers.h\n",
+            builtin_ref_counter, EXPECTED_PRIM_COUNT);
     abort();
   }
 #endif
@@ -370,6 +345,7 @@ static void init_unsafe(Scheme_Startup_Env *env)
   scheme_init_unsafe_number(env);
   scheme_init_unsafe_numarith(env);
   scheme_init_unsafe_numcomp(env);
+  scheme_init_unsafe_char(env);
   scheme_init_unsafe_list(env);
   scheme_init_unsafe_hash(env);
   scheme_init_unsafe_vector(env);
@@ -445,6 +421,10 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
 
   scheme_init_compenv_places();
 
+#ifdef MZ_USE_JIT
+  scheme_init_jitprep();
+#endif
+
 #ifdef TIME_STARTUP_PROCESS
   printf("pre-process @ %" PRIdPTR "\n", scheme_get_process_milliseconds());
 #endif
@@ -496,9 +476,7 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
   scheme_init_port_fun_config();
   scheme_init_error_config();
   scheme_init_logger_config();
-#ifndef NO_SCHEME_EXNS
   scheme_init_exn_config();
-#endif
   scheme_init_error_config();
   scheme_init_place_per_place();
 
@@ -532,6 +510,8 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
 
   scheme_starting_up = 0;
 
+  scheme_performance_record_end("boot", NULL);
+
   --scheme_current_thread->suspend_break; /* created with breaks suspended */
 
 #ifdef TIME_STARTUP_PROCESS
@@ -554,6 +534,7 @@ Scheme_Env *scheme_place_instance_init(void *stack_base, struct NewGC *parent_gc
   GC_construct_child_gc(parent_gc, memory_limit);
 # endif
   scheme_rktio = rktio_init();
+  if (!scheme_rktio) return NULL;
   env = place_instance_init(stack_base, 0);
 # if defined(MZ_PRECISE_GC)
   if (scheme_rktio) {
@@ -603,6 +584,7 @@ void scheme_place_instance_destroy(int force)
   GC_destruct_child_gc();
 #endif
   scheme_free_all_code();
+  scheme_clear_locale_cache();
   rktio_destroy(scheme_rktio);
 }
 
